(ns rlm.iterate
  "from the clojure mailing list"
  {:author "tsuraan"}
  (:gen-class
   :name "iterate"
   :main true
   ))


(set! *warn-on-reflection* true)
(import java.io.FileInputStream)

(def *numbytes* (* 16 1024 1024))


(defmacro better-areduce [a idx ret init expr]
  `(let [a# ~a
	 alength# (alength a#)]
     (loop [~idx (int 0) ~ret ~init]
       (if (< ~idx alength#)
	 (recur (unchecked-inc ~idx) ~expr)
	 ~ret))))

(defn translated-countnl
  [#^bytes buf]
  (let [total-length (alength buf)]
    (loop [count (int 0) current-index (int 0)]
      (if (>= current-index total-length)
	count
	(let [new-count
	      (if (= (aget #^bytes buf current-index) (byte 10))
		(unchecked-inc count)
		count)]
	  
	  (recur new-count (unchecked-inc current-index))))))) 
	 
(defn countnl-classic
  [#^bytes buf]
  (let [nl (byte 10)]
    (areduce buf idx count 0
	     (if (= (aget buf idx) nl)
	       (inc count)
	       count))))


(defn countnl-lite
  "this is the absolute fastest function I could come up with.
   still 2.25 times slower that the java version."
  [#^bytes buf]
  (areduce buf idx count (int 0)
	   (if (= (clojure.lang.RT/aget buf idx) 10)
	     (unchecked-add count 1)
	     count)))


(defn #^Boolean is-ten? [#^Integer A]
     (= A 10))

(defn countnl-max-power
  [#^bytes buf]
  (areduce buf idx count (int 0)
	   (if (is-ten? (clojure.lang.RT/aget buf idx))
	     (unchecked-inc count)
	     count)))




			 
			    

			  
	     


"(times are averages with 10 rounds)
no-modifications 
   classic:  198.151
   lite:     186.668

unchecked-inc
   classic:  177.938
   lite:     129.604

inlined-let
   classic:  202.604
   lite:     195.357

make nl a constant
   classic:
   lite:     361.734

make nl a constant with type hint
   lite:     394.45

literal 10, no let
   lite:     102.876

 (int 10) instead of literal 10
   lite:     194.904

 (unchecked-add count 0) instead of count
   lite:     209.186

 direct reference to RT/aget, bypassing int cast
   lite:      99.767

 (int 0) instead of 0 initialization of count
   lite:      66.446

 make nl an (int 10) instead of (byte 10)
   lite:     180.003 
"





(defmacro time-multi
  "Evaluates expr and prints the time it took.  Returns the value of
 expr."
  {:added "1.0"}
  [expr]
  `(let [start# (. System (nanoTime))
         ret# ~expr
	 end# (/ (double (- (. System (nanoTime)) start#)) 1000000.0)]
     (prn (str "Elapsed time: " end# " msecs"))
     [ret# end#]))



(defn run [countnl-impl]
  (let [ifs (FileInputStream. "/dev/urandom")
	buf (make-array Byte/TYPE *numbytes*)
	runs 10]
    (loop [n 0 total-time 0]
      (if (= n runs) (/ total-time runs)
	  (let [sz (.read #^FileInputStream ifs buf)]
	    (println "Wanted" *numbytes* "got" sz "bytes")
	    (let [[count time] (time-multi (countnl-impl buf))]
	      (println "Got" count "nls")
	      (recur (inc n) (+ total-time time))))))))

(defn -main [& ignore]
  (run countnl-lite)
  (run countnl-classic)
  )


  
