(ns rlm.thermal-noise
  (:refer-clojure :only [])
  (:require rlm.ns-rlm mobius.base))
(rlm.ns-rlm/ns-clone mobius.base)

(defrecord imaginary [real complex])

(defmethod + [imaginary Number] [i r]
	   (imaginary. (+ (:real i) r) (:complex i)))
(defmethod + [Number imaginary] [r i]
	   (+ i r))
(defmethod + [imaginary imaginary] [i1 i2]
	   (imaginary. (+ (:real i1) (:real i2))
		       (+ (:complex i1) (:complex i2))))


(defmethod * [imaginary Number] [i r]
	   (imaginary. (* r (:real i)) (* r (:complex i))))
(defmethod * [Number imaginary] [r i]
	   (* i r))
(defmethod * [imaginary imaginary] [i1 i2]
	   (let [[a1 b1] [(:real i1) (:complex i1)]
		 [a2 b2] [(:real i2) (:complex i2)]]
	     (imaginary. (- (* a1 a2) (* b1 b2))
			 (+ (* a1 b2) (* a2 b1)))))

(defmethod / imaginary [i]

	   (cond (and (not= (:real i) 0) (not= (:complex i) 0)) 
		 (let [[a b] [(:real i) (:complex i)]
		       d (- (/ (/ a) (+ (/ a b) (/ b a))))
		       c (/ (+ 1 (* b d)) a)]
		   (imaginary. c d))
		 (= (:real i) 0)
		 (imaginary. 0 (- (/ (:complex i))))
		 (= (:complex i) 0)
		 (imaginary. (/ (:real i)) 0)))
		 
	   
(defmethod / [imaginary imaginary] [i1 i2]
	   (* i1 (/ i2)))

(defmethod / [imaginary Number] [i r]
	   (* i (/ r)))

(defmethod / [Number imaginary] [r i]
	   (* r (/ i)))

(defmulti norm class)
(defmethod norm Number [r] r)
(defmethod norm imaginary [i]
	   (sqrt (+ (sq (:real i) ) (sq (:complex i)))))
	   



(def i (imaginary. 0 1))

(defn h** [w_0 v_0 y w]
  (/ v_0 (+ (- (sq w)) (* 2 y i w) (sq w_0))))

(def f_0 10e3)
(def w_0 (* f_0 2 Math/PI))
(def v_0 0.01)
(def h* (partial h** w_0 v_0))


(def x (range 1 1e7 100))

(def y-range [0.1 0.5 1 2 5 10 100 1000])


(defn graphs []
  (let [plot (xy-plot)]
    (dorun
     (for [ y y-range]
       (let [result (vec (map (partial (comp norm h*) y) x))]
	 (add-lines plot (vec (map log10 x)) (vec (map log10 result))))))
     (set-x-label plot "log frequency")
     (set-y-label plot "log h*")))

(def rrr (imaginary. 1 1))


