(ns rlm.visualize
  "this namespace has only one purpose -- to enable
   visual display of as many data structures as possible,
   with a focus on image processing (for example, raw integers
   will be shown as the color which they would encode in a RGBA
   BufferedImage.  I want to get as many types as I can in here."
  {:author "Robert McIntyre"}

  (:import java.awt.image.BufferedImage
	   [ij ImagePlus IJ]))

(defmulti visual (fn [& args] (class (last args))))

(import '[org.scilab.forge.jlatexmath TeXConstants TeXFormula TeXIcon])
(import java.awt.Insets)
(import javax.swing.JLabel)
(import java.awt.Color)


(defmethod visual ImagePlus
  [image]
  (.show image)
  image)
    
(defmethod visual (class 4)
  [color]
  (let [image (BufferedImage. 200 200 BufferedImage/TYPE_INT_RGB)]
    (doall (for [y (range 200) x (range 200)] (.setRGB image x y color)))
    (visual (ImagePlus. "color display" image))))
  
(defmethod visual BufferedImage
  [image]
  (visual (ImagePlus. "visual" image)))
    

(defmethod visual TeXFormula
  [formula]
  (let [icon
	(doto (.createTeXIcon formula TeXConstants/STYLE_DISPLAY 30)
	  (.setInsets (Insets. 5 5 5 5)))
	image (BufferedImage. (.getIconWidth icon) (.getIconHeight icon)
			      BufferedImage/TYPE_INT_ARGB)
	g (.createGraphics image)
	jl (JLabel.)]
    (.setForeground jl (Color. 0 0 0))
    (.setColor g Color/white)
    (.fillRect g 0 0 (.getIconWidth icon) (.getIconHeight icon))
    (.paintIcon icon jl g 0 0)
    (visual image)))
	
    
	
	
    
   
	   
  
